@@ -16,7 +16,7 @@ from .notifier import Notifier, notifier
 from .named_expr import NamedExpression, B, L, C, F, V, M, D, R, W, P, PP, I, eval_expr
 from .plotter import plot
 from .dsindex import DatasetIndex, FilesIndex
-from .decorators import action, inbatch_parallel, parallel, any_action_failed, mjit, deprecated, apply_parallel
+from .decorators import action, any_action_failed, mjit, deprecated, apply_parallel
 from .exceptions import SkipBatchException, EmptyBatchSequence, StopPipeline
 from .run_notebook import run_notebook
 from .sampler import Sampler, ConstantSampler, NumpySampler, HistoSampler, ScipySampler
@@ -27,4 +27,4 @@ from .utils_notebook import in_notebook, get_notebook_path, get_notebook_name, p
 from .utils_telegram import TelegramMessage
 
 
-__version__ = '0.7.5'
+__version__ = '0.7.6'
