@@ -331,15 +331,17 @@ class ImagesBatch(BaseImagesBatch):
                 origin = (np.random.randint(background_shape[0]-image_shape[0]+1),
                           np.random.randint(background_shape[1]-image_shape[1]+1))
             else:
-                raise ValueError("If string, origin should be one of ['center', 'top_left', 'top_right', "
-                                 "'bottom_left', 'bottom_right', 'random']. Got '{}'.".format(origin))
+                msg = "If string, origin should be one of ['center', 'top_left', 'top_right', "\
+                      f"'bottom_left', 'bottom_right', 'random']. Got '{origin}'."
+                raise ValueError(msg)
         elif all(0 <= elem < 1 for elem in origin):
             region = ((background_shape[0]-image_shape[0]+1),
                       (background_shape[1]-image_shape[1]+1))
             origin = np.asarray(origin) * region
         elif not all(isinstance(elem, int) for elem in origin):
-            raise ValueError('If not a string, origin should be either a sequence of ints or sequence of '
-                             'floats in [0, 1) interval. Got {}'.format(origin))
+            msg = "If not a string, origin should be either a sequence of ints or "\
+                  f"sequence of floats in [0, 1) interval. Got {origin}"
+            raise ValueError(msg)
 
         return np.asarray(origin, dtype=np.int)
 
