@@ -393,15 +393,69 @@ class ConvNeXtBlock(Block):
         - Label Smoothing
         - AdamW, EMA
     """
-    def __init__(self, inputs=None, layout='Rwlcac!', channels='same', expand_ratio=4, kernel_size=7, stride=1,
-                 drop_path=0.0, layer_scale=1e-6, **kwargs):
+    def __init__(self, inputs=None, layout='Rwncac!', channels='same', expand_ratio=4, kernel_size=7, stride=1,
+                 drop_path=0.0, layer_scale=1, **kwargs):
         channels = safe_eval(channels, get_num_channels(inputs))
 
-        super().__init__(inputs=inputs, layout=layout, activation='GELU',
-                         channels=[channels, channels * expand_ratio, channels],
-                         stride=[stride, 1, 1],
-                         bias=True,
-                         kernel_size=[kernel_size, 1, 1],
-                         layer_norm={'data_format':'channels_first'},
-                         branch_end={'drop_path': drop_path, 'layer_scale': layer_scale},
-                         **kwargs)
+        kwargs = {
+            'activation': 'GELU',
+            'channels': [channels, channels * expand_ratio, channels],
+            'stride': [stride, 1, 1],
+            'bias': True,
+            'kernel_size': [kernel_size, 1, 1],
+            'normalization_type': 'instance',
+            'branch_end': {'drop_path': drop_path, 'layer_scale': layer_scale},
+            **kwargs
+        }
+        super().__init__(inputs=inputs, layout=layout, **kwargs)
+
+
+class MSCANBlock(Block):
+    """ MultiScaleConv block: modern Inception-like block with some adaptation from Transformers.
+    Meng-Hao Guo et al. "`SegNeXt: Rethinking convolutional attention design for Semantic Segmentation
+    <https://arxiv.org/abs/2209.08575v1>`_"
+
+    In the default case, the layout is:
+        Rnca Rc Rm+ c* c+ Rnccac+
+        │    │  └0┘  │  │ └──3──┘
+        │    └───1───┘  │
+        └───────2───────┘
+        - `0` is multi scale conv: multiple factorized kernels, combined into one tensor.
+        - `1` uses multi-scale conv as re-weighting for input tensor.
+        - `2` adds normalization and activation layers.
+        - `3` stacks an MLP block, inspired by recent transformers.
+
+    Parameters
+    ----------
+    msca_kernel_size : sequence of ints
+        Kernel sizes in multi-scale convolution.
+    mlp : bool
+        Whether to stack an additional MLP block on top.
+    mlp_expansion : int
+        Expansion ratio for channels in MLP block.
+    """
+    def __init__(self, inputs=None, layout='Rnca (Rc Rm! c*) c!', msca_kernel_size=(7, 11, 15),
+                 mlp=True, mlp_expansion=4, drop_path=0.0, layer_scale=1, **kwargs):
+        in_channels = get_num_channels(inputs)
+
+        channels = [in_channels] * 5
+        kernel_size = [1, 5, list(msca_kernel_size), 1, 1]
+        groups = [1, in_channels, in_channels, 1, 1]
+
+        if mlp:
+            layout = layout + 'Rnccac!'
+            channels.extend([in_channels, in_channels*mlp_expansion, in_channels])
+            kernel_size.extend([1, 3, 1])
+            groups.extend([1, in_channels, 1])
+
+        kwargs = {
+            'channels': channels,
+            'stride': 1,
+            'bias': True,
+            'kernel_size': kernel_size,
+            'groups': groups,
+            'activation': 'GELU',
+            'branch_end': {'drop_path': drop_path, 'layer_scale': layer_scale},
+            **kwargs
+        }
+        super().__init__(inputs=inputs, layout=layout, **kwargs)
